/*

Resizable Form Fields
Justin Watt
justincwatt@gmail.com
http://justinsomnia.org/

based on:

Resize Textarea 0.1
Original coding by Raik Juergens
Contact: borstel33@web.de

CHANGELOG

v0.2.4
fix multiple tabs resizing conflict by recreating element references on activate (thanks John)

v0.2.3
removed corner and horizonal grip from select box
reduced grip width from 5 pixels to 4, removed that one pixel from within the widget
prevent resizing width and height less than 20 pixels
stop creating grips for file inputs
changed spelling of Resizeable to Resizable

v0.2.2
set z-index style property of resize grips to 9999, to position above other absolutely positioned content (slashdot bug pointed out by Dave Derrick)
added ability to resize iframes
removed corner grip from input fields (suggested by Jesse Pelton and Archaeopteryx)
code cleanup

v0.2.1
added ability to resize input boxes with type unspecified---which defaults to text (thanks Jesse Pelton)
removed vertical grip from input boxes (which can't/shouldn't be resized vertically)

v0.2
added ability to resize select boxes and input boxes where type="text"
repackaged plugin with chrome.manifest for FF 1.5-2.0

v0.1
original version by Raik Juergens

*/

window.addEventListener("load",function() { resizeTa.init(); },false);

var resizeTa =
{
    loaded: null,
    TAlength: 0,
    TA: new Array(),

    init: function () {
        if (resizeTa.loaded) {
            return;
        } else {
            resizeTa.loaded = true;
            var appcontent = document.getElementById("appcontent");
            if (appcontent) {
                appcontent.addEventListener("load", resizeTa.pageload, true);
            }
        }
    },

    collectionToArray: function (col) {
        a = new Array();
        for (i = 0; i < col.length; i++)
            a[a.length] = col[i];
        return a;
    },

    load: function () {
    
        resizeTa.TA = resizeTa.collectionToArray(resizeTa.doc.getElementsByTagName('TEXTAREA'));
        resizeTa.TA = resizeTa.TA.concat(resizeTa.collectionToArray(resizeTa.doc.getElementsByTagName('SELECT')));
        resizeTa.TA = resizeTa.TA.concat(resizeTa.collectionToArray(resizeTa.doc.getElementsByTagName('IFRAME')));
        
        var inputs;
        inputs = resizeTa.doc.getElementsByTagName('INPUT');
        for (var i = 0; i < inputs.length; i++) {
            if (!inputs[i].hasAttribute('type') ||
                 inputs[i].getAttribute('type') == 'text' ||
                 inputs[i].getAttribute('type') == 'password') {
                resizeTa.TA.push(inputs[i]);
            }
        }


    },

    pageload: function (aEvent) {
        resizeTa.doc = aEvent.originalTarget;

        resizeTa.load();

        resizeTa.TAlength = resizeTa.TA.length;
        if (resizeTa.TAlength == 0) {
            return;
        } else {
            resizeTa.rootElem = resizeTa.doc.getElementsByTagName('HTML')[0];
            var i = resizeTa.TAlength;
            while (i--) {
                if (resizeTa.TA[i].tagName.toLowerCase() == "textarea" ||
                    resizeTa.TA[i].tagName.toLowerCase() == "iframe") {
                    resizeTa.newdiv('4' ,'1' ,'gripH_',i,'w' );
                    resizeTa.newdiv('1' ,'4' ,'gripV_',i,'n' );
                    resizeTa.newdiv('10','10','gripX_',i,'se');
                
                } else if (resizeTa.TA[i].tagName.toLowerCase() == "select") {
                    resizeTa.newdiv('1' ,'4' ,'gripV_',i,'n' );
                
                } else if (resizeTa.TA[i].tagName.toLowerCase() == "input") {
                    resizeTa.newdiv('4' ,'1' ,'gripH_',i,'w' );
                }

            }
            resizeTa.newdiv('0','0','showCursor','','w');
            CursorDiv = resizeTa.doc.getElementById('showCursor');
            CursorDiv.removeEventListener('mousedown', resizeTa.activate, true);
            CursorDiv.style.left = '0px';
            CursorDiv.style.top  = '0px';
            resizeTa.posdivs();
            window.addEventListener("resize",resizeTa.posdivs,true);
        }
    },

    newdiv: function (w,h,id,nr,cu) {
        var grip = resizeTa.doc.createElement("div");
        grip.setAttribute("ID", id+nr);
        grip.setAttribute("STYLE", "position:absolute;z-index:9999;width:"+w+"px;height:"+h+"px;cursor:"+cu+"-resize;");
        grip.addEventListener('mousedown', resizeTa.activate, true);
        resizeTa.rootElem.appendChild(grip);
    },

    getposition: function (i) {
        var curElem = resizeTa.TA[i];
        var curX = curElem.offsetLeft;
        while (curElem.offsetParent) {
            curX += curElem.offsetParent.offsetLeft;
            curElem = curElem.offsetParent;
        }
        curElem = resizeTa.TA[i];
        var curY = curElem.offsetTop;
        while (curElem.offsetParent) {
            curY += curElem.offsetParent.offsetTop;
            curElem = curElem.offsetParent;
        }
        return [curX,curY]
    },

    posdivs: function () {
        var k = resizeTa.TAlength;
        while (k--) {
            curPos = resizeTa.getposition(k);
            
            // only add horizontal grip to input boxes
            // note: height is different due to absense of corner grip
            if (resizeTa.TA[k].tagName.toLowerCase() == "input") {
                resizeTa.doc.getElementById('gripH_'+k).style.left   = curPos[0]+resizeTa.TA[k].offsetWidth -2 + "px";
                resizeTa.doc.getElementById('gripH_'+k).style.top    = curPos[1]                               + "px";
                resizeTa.doc.getElementById('gripH_'+k).style.height =           resizeTa.TA[k].offsetHeight   + "px";
            }
            
            // only add vertical grip to select box
            if (resizeTa.TA[k].tagName.toLowerCase() == "select") {
                resizeTa.doc.getElementById('gripV_'+k).style.left   = curPos[0]                                + "px";
                resizeTa.doc.getElementById('gripV_'+k).style.top    = curPos[1]+resizeTa.TA[k].offsetHeight-2  + "px";
                resizeTa.doc.getElementById('gripV_'+k).style.width  =           resizeTa.TA[k].offsetWidth -18 + "px";            
            }
            
            // add horizontal, vertical, and corner grippies to textarea and select
            if (resizeTa.TA[k].tagName.toLowerCase() == "textarea" ||
                resizeTa.TA[k].tagName.toLowerCase() == "iframe") {
                resizeTa.doc.getElementById('gripH_'+k).style.left   = curPos[0]+resizeTa.TA[k].offsetWidth -2 + "px";
                resizeTa.doc.getElementById('gripH_'+k).style.top    = curPos[1]                               + "px";
                resizeTa.doc.getElementById('gripH_'+k).style.height =           resizeTa.TA[k].offsetHeight-8 + "px";        
            
                resizeTa.doc.getElementById('gripV_'+k).style.left   = curPos[0]                               + "px";
                resizeTa.doc.getElementById('gripV_'+k).style.top    = curPos[1]+resizeTa.TA[k].offsetHeight-2 + "px";
                resizeTa.doc.getElementById('gripV_'+k).style.width  =           resizeTa.TA[k].offsetWidth -8 + "px";
            
                resizeTa.doc.getElementById('gripX_'+k).style.left   = curPos[0]+resizeTa.TA[k].offsetWidth -8 + "px";
                resizeTa.doc.getElementById('gripX_'+k).style.top    = curPos[1]+resizeTa.TA[k].offsetHeight-8 + "px";
            }
        }
    },

    activate: function (e) {
        resizeTa.doc    = e.target.ownerDocument;
        resizeTa.load();
        CursorDiv       = resizeTa.doc.getElementById('showCursor');
        var curTargetId = e.target.getAttribute('ID').split("_");
        curTarget       = curTargetId[0];
        curTA_Nr        = parseInt(curTargetId[1]);
        resizeTa.doc.addEventListener('mouseup', resizeTa.deactivate, true);
        switch (curTarget) {
            case "gripH": resizeTa.doc.addEventListener('mousemove', resizeTa.resizeta_h, true); break;
            case "gripV": resizeTa.doc.addEventListener('mousemove', resizeTa.resizeta_v, true); break;
            case "gripX": resizeTa.doc.addEventListener('mousemove', resizeTa.resizeta_x, true); break;
        }
        CursorDiv.style.width  = resizeTa.rootElem.offsetWidth + "px";
        CursorDiv.style.height = resizeTa.rootElem.offsetHeight + "px";
        CursorDiv.style.cursor = e.target.style.cursor;
    },

    deactivate: function () {
        resizeTa.doc.removeEventListener('mouseup', resizeTa.deactivate, true);
        switch (curTarget) {
            case "gripH": resizeTa.doc.removeEventListener('mousemove', resizeTa.resizeta_h, true); break;
            case "gripV": resizeTa.doc.removeEventListener('mousemove', resizeTa.resizeta_v, true); break;
            case "gripX": resizeTa.doc.removeEventListener('mousemove', resizeTa.resizeta_x, true); break;
        }
        CursorDiv.style.width  = '0px';
        CursorDiv.style.height = '0px';
        resizeTa.posdivs();
    },

    resizeta_h: function (e) {
        curPos = resizeTa.getposition(curTA_Nr);
        if (e.pageX - curPos[0] > 20) {
            resizeTa.TA[curTA_Nr].style.width = e.pageX - curPos[0] + "px";
        }
    },

    resizeta_v: function (e) {
        curPos = resizeTa.getposition(curTA_Nr);
        if (e.pageY - curPos[1] > 20) {
            resizeTa.TA[curTA_Nr].style.height = e.pageY - curPos[1] + "px";
        }
        if (resizeTa.TA[curTA_Nr].tagName.toLowerCase() == "select") {
            if (e.pageY - curPos[1] < 25) {
                resizeTa.TA[curTA_Nr].setAttribute("size", 1);
            } else {
                resizeTa.TA[curTA_Nr].setAttribute("size", 2);
            }
        }
    },

    resizeta_x: function (e) {
        curPos = resizeTa.getposition(curTA_Nr);
        
        if (e.pageX - curPos[0] > 20 && e.pageY - curPos[1] > 20) {
            resizeTa.TA[curTA_Nr].style.width  = e.pageX - curPos[0] + 2 + "px";
            resizeTa.TA[curTA_Nr].style.height = e.pageY - curPos[1] + 2 + "px";
        }
        if (resizeTa.TA[curTA_Nr].tagName.toLowerCase() == "select") {
            resizeTa.TA[curTA_Nr].setAttribute("size", 2);
        }
    }
}
